@@ -1,10 +1,10 @@
 #!/usr/bin/env python3
 '''
-remove_objects.py - this file is part of S3QL (http://s3ql.googlecode.com)
+remove_objects.py - this file is part of S3QL.
 
 Copyright © 2014 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 import os
@@ -52,7 +52,7 @@ def main(args=None):
     options = parse_args(args)
     setup_logging(options)
 
-    backend = get_backend(options, plain=True)
+    backend = get_backend(options, raw=True)
     atexit.register(backend.close)
 
     for line in options.file:
