@@ -1,30 +1,28 @@
 '''
-adm.py - this file is part of S3QL (http://s3ql.googlecode.com)
+adm.py - this file is part of S3QL.
 
 Copyright © 2008 Nikolaus Rath <Nikolaus@rath.org>
 
-This program can be distributed under the terms of the GNU GPLv3.
+This work can be distributed under the terms of the GNU GPLv3.
 '''
 
 from .logging import logging, QuietError, setup_logging
-from . import CURRENT_FS_REV, REV_VER_MAP, PICKLE_PROTOCOL, BUFSIZE
+from . import CURRENT_FS_REV, REV_VER_MAP
 from .backends.comprenc import ComprencBackend
 from .database import Connection
-from .common import (get_backend_cachedir, get_seq_no, stream_write_bz2,
-                     stream_read_bz2, is_mounted, AsyncFn, get_backend,
-                     get_backend_factory, pretty_print_size, split_by_n,
-                     handle_on_return)
-from .metadata import restore_metadata, cycle_metadata, dump_metadata
+from .deltadump import TIME, INTEGER
+from .common import (get_backend_cachedir, get_seq_no, is_mounted, get_backend,
+                     load_params, save_params)
+from .metadata import dump_and_upload_metadata, download_metadata
+from . import metadata
 from .parse_args import ArgumentParser
 from datetime import datetime as Datetime
 from getpass import getpass
-from base64 import b64encode
-from queue import Queue, Full as QueueFull
+from contextlib import contextmanager
 import os
-import pickle
 import shutil
+import functools
 import sys
-import tempfile
 import textwrap
 import time
 
@@ -66,7 +64,6 @@ def parse_args(args):
     parser.add_backend_options()
     parser.add_cachedir()
     parser.add_version()
-    parser.add_fatal_warnings()
 
     options = parser.parse_args(args)
 
@@ -87,20 +84,20 @@ def main(args=None):
         raise QuietError('Can not work on mounted file system.')
 
     if options.action == 'clear':
-        with get_backend(options, plain=True) as backend:
+        with get_backend(options, raw=True) as backend:
             return clear(backend, options)
 
-    if options.action == 'upgrade':
-        return upgrade(options)
-    
     with get_backend(options) as backend:
-        if options.action == 'passphrase':
+        if options.action == 'upgrade':
+            return upgrade(backend, get_backend_cachedir(options.storage_url,
+                                                          options.cachedir))
+        elif options.action == 'passphrase':
             return change_passphrase(backend)
 
-        if options.action == 'download-metadata':
-            return download_metadata(backend, options.storage_url)
+        elif options.action == 'download-metadata':
+            return download_metadata_cmd(backend, options.storage_url)
 
-def download_metadata(backend, storage_url):
+def download_metadata_cmd(backend, storage_url):
     '''Download old metadata backups'''
 
     backups = sorted(backend.list('s3ql_metadata'))
@@ -139,25 +136,13 @@ def download_metadata(backend, storage_url):
             raise QuietError('%s already exists, aborting.' % cachepath + i)
 
     param = backend.lookup(name)
-    with tempfile.TemporaryFile() as tmpfh:
-        def do_read(fh):
-            tmpfh.seek(0)
-            tmpfh.truncate()
-            stream_read_bz2(fh, tmpfh)
-
-        log.info('Downloading and decompressing %s...', name)
-        backend.perform_read(do_read, name)
-
-        log.info("Reading metadata...")
-        tmpfh.seek(0)
-        restore_metadata(tmpfh, cachepath + '.db')
+    download_metadata(backend, cachepath + ".db", name)
 
     # Raise sequence number so that fsck.s3ql actually uses the
     # downloaded backup
     seq_nos = [ int(x[len('s3ql_seq_no_'):]) for x in backend.list('s3ql_seq_no_') ]
     param['seq_no'] = max(seq_nos) + 1
-    with open(cachepath + '.params', 'wb') as fh:
-        pickle.dump(param, fh, PICKLE_PROTOCOL)
+    save_params(cachepath, param)
 
 def change_passphrase(backend):
     '''Change file system passphrase'''
@@ -216,28 +201,21 @@ def get_old_rev_msg(rev, prog):
           $ wget http://s3ql.googlecode.com/files/s3ql-%(version)s.tar.bz2 \
             || wget https://bitbucket.org/nikratio/s3ql/downloads/s3ql-%(version)s.tar.bz2
           $ tar xjf s3ql-%(version)s.tar.bz2
-          $ (cd s3ql-%(version)s; ./setup.py build_ext)
+          $ (cd s3ql-%(version)s; ./setup.py build_ext --inplace)
           $ s3ql-%(version)s/bin/%(prog)s <options>
         ''' % { 'version': REV_VER_MAP[rev],
                 'prog': prog })
 
-
-@handle_on_return
-def upgrade(options, on_return):
+def upgrade(backend, cachepath):
     '''Upgrade file system to newest revision'''
 
     log.info('Getting file system parameters..')
-    cachepath = get_backend_cachedir(options.storage_url, options.cachedir)
-
-    backend_factory = get_backend_factory(options)
-    backend = on_return.enter_context(backend_factory())
 
     # Check for cached metadata
     db = None
     seq_no = get_seq_no(backend)
     if os.path.exists(cachepath + '.params'):
-        with open(cachepath + '.params', 'rb') as fh:
-            param = pickle.load(fh)
+        param = load_params(cachepath)
         if param['seq_no'] < seq_no:
             log.info('Ignoring locally cached metadata (outdated).')
             param = backend.lookup('s3ql_metadata')
@@ -304,19 +282,8 @@ def upgrade(options, on_return):
         raise QuietError()
 
     if not db:
-        # Need to download metadata
-        with tempfile.TemporaryFile() as tmpfh:
-            def do_read(fh):
-                tmpfh.seek(0)
-                tmpfh.truncate()
-                stream_read_bz2(fh, tmpfh)
-
-            log.info("Downloading & uncompressing metadata...")
-            backend.perform_read(do_read, "s3ql_metadata")
-
-            log.info("Reading metadata...")
-            tmpfh.seek(0)
-            db = restore_metadata(tmpfh, cachepath + '.db')
+        with monkeypatch_metadata_retrieval():
+            db = download_metadata(backend, cachepath + '.db')
 
     log.info('Upgrading from revision %d to %d...', param['revision'], CURRENT_FS_REV)
 
@@ -324,196 +291,65 @@ def upgrade(options, on_return):
     param['last-modified'] = time.time()
     param['seq_no'] += 1
 
-    # Ensure that there are backups of the master key
-    if backend.passphrase is not None:
-        data_pw = backend.passphrase
-        backend.passphrase = backend.fs_passphrase
-        for i in range(1,4):
-            obj_id = 's3ql_passphrase_bak%d' % i
-            if obj_id not in backend:
-                backend[obj_id] = data_pw
-        backend.passphrase = data_pw
-
-    # Upgrade all objects, so that we can remove legacy conversion
-    # routines in the next release.
-    update_obj_metadata(backend, backend_factory, db, options.threads)
-
-    log.info('Dumping metadata...')
-    with tempfile.TemporaryFile() as fh:
-        dump_metadata(db, fh)
-        def do_write(obj_fh):
-            fh.seek(0)
-            stream_write_bz2(fh, obj_fh)
-            return obj_fh
-
-        log.info("Compressing and uploading metadata...")
-        backend.store('s3ql_seq_no_%d' % param['seq_no'], b'Empty')
-        obj_fh = backend.perform_write(do_write, "s3ql_metadata_new", metadata=param,
-                                      is_compressed=True)
-
-    log.info('Wrote %s of compressed metadata.', pretty_print_size(obj_fh.get_obj_size()))
-    log.info('Cycling metadata backups...')
-    cycle_metadata(backend)
+    # Upgrade
+    for name in ('atime', 'mtime', 'ctime'):
+        db.execute('ALTER TABLE inodes ADD COLUMN {time}_ns '
+                   'INT NOT NULL DEFAULT 0'.format(time=name))
+        db.execute('UPDATE inodes SET {time}_ns = {time} * 1e9'.format(time=name))
 
+    dump_and_upload_metadata(backend, db, param)
     backend['s3ql_seq_no_%d' % param['seq_no']] = b'Empty'
 
-    with open(cachepath + '.params', 'wb') as fh:
-        pickle.dump(param, fh, PICKLE_PROTOCOL)
-
-    log.info('Cleaning up local metadata...')
-    db.execute('ANALYZE')
-    db.execute('VACUUM')
+    # Declare local metadata as outdated so that it won't be used. It still
+    # contains the old [acm]time columns which are NON NULL, and would prevent
+    # us from inserting new rows.
+    param['seq_no'] = 0
+    save_params(cachepath, param)
 
     print('File system upgrade complete.')
 
-    if backend.passphrase is not None:
-        print('Please store the following master key in a safe location. It allows ',
-              'decryption of the S3QL file system in case the storage objects holding ',
-              'this information get corrupted:',
-              '---BEGIN MASTER KEY---',
-              ' '.join(split_by_n(b64encode(backend.passphrase).decode(), 4)),
-              '---END MASTER KEY---',
-              sep='\n')
 
-@handle_on_return
-def update_obj_metadata(backend, backend_factory, db,
-                        thread_count, on_return):
-    '''Upgrade metadata of storage objects'''
+@contextmanager
+def monkeypatch_metadata_retrieval():
+    create_tables_bak = metadata.create_tables
+    @functools.wraps(metadata.create_tables)
+    def create_tables(conn):
+        create_tables_bak(conn)
+        conn.execute('DROP TABLE inodes')
+        conn.execute("""
+           CREATE TABLE inodes (
+           id        INTEGER PRIMARY KEY AUTOINCREMENT,
+           uid       INT NOT NULL,
+           gid       INT NOT NULL,
+           mode      INT NOT NULL,
+           mtime     REAL NOT NULL,
+           atime     REAL NOT NULL,
+           ctime     REAL NOT NULL,
+           refcount  INT NOT NULL,
+           size      INT NOT NULL DEFAULT 0,
+           rdev      INT NOT NULL DEFAULT 0,
+           locked    BOOLEAN NOT NULL DEFAULT 0
+        )""")
+    metadata.create_tables = create_tables
 
-    plain_backend = backend.backend
+    DUMP_SPEC_bak = metadata.DUMP_SPEC[2]
+    metadata.DUMP_SPEC[2] = ('inodes', 'id', (('id', INTEGER, 1),
+                                              ('uid', INTEGER),
+                                              ('gid', INTEGER),
+                                              ('mode', INTEGER),
+                                              ('mtime', TIME),
+                                              ('atime', TIME),
+                                              ('ctime', TIME),
+                                              ('size', INTEGER),
+                                              ('rdev', INTEGER),
+                                              ('locked', INTEGER),
+                                              ('refcount', INTEGER)))
 
-    # No need to update sequence number, since we are going to
-    # write out a new one after the upgrade.
-    if backend.passphrase is None:
-        extra_objects = { 's3ql_metadata' }
-    else:
-        extra_objects = { 's3ql_metadata',
-                          's3ql_passphrase', 's3ql_passphrase_bak1',
-                          's3ql_passphrase_bak2', 's3ql_passphrase_bak3' }
-
-    for i in range(30):
-        obj_id = 's3ql_metadata_bak_%d' % i
-        if obj_id in plain_backend:
-            extra_objects.add(obj_id)
-
-    def yield_objects():
-        for (id_,) in db.query('SELECT id FROM objects'):
-            yield 's3ql_data_%d' % id_
-        for obj_id in extra_objects:
-            yield obj_id
-    total = db.get_val('SELECT COUNT(id) FROM objects') + len(extra_objects)
-
-    queue = Queue(maxsize=thread_count)
-    threads = []
-    for _ in range(thread_count):
-        t = AsyncFn(upgrade_loop, queue, on_return.push(backend_factory()))
-        # Don't wait for worker threads, gives deadlock if main thread
-        # terminates with exception
-        t.daemon = True
-        t.start()
-        threads.append(t)
-
-    # Updating this value is prone to race conditions. However,
-    # we don't care because this is for an approximate progress
-    # output only.
-    queue.rewrote_size = 0
-    stamp = 0
-    for (i, obj_id) in enumerate(yield_objects()):
-        stamp2 = time.time()
-        if stamp2 - stamp > 1:
-            sys.stdout.write('\r..processed %d/%d objects (%.1f%%, %s rewritten)..'
-                             % (i, total, i/total*100,
-                                pretty_print_size(queue.rewrote_size)))
-            sys.stdout.flush()
-            stamp = stamp2
-
-            # Terminate early if any thread failed with an exception
-            for t in threads:
-                if not t.is_alive():
-                    t.join_and_raise()
-
-        # Avoid blocking if all threads terminated
-        while True:
-            try:
-                queue.put(obj_id, timeout=1)
-            except QueueFull:
-                pass
-            else:
-                break
-            for t in threads:
-                if not t.is_alive():
-                    t.join_and_raise()
-
-    queue.maxsize += len(threads)
-    for t in threads:
-        queue.put(None)
-
-    for t in threads:
-        t.join_and_raise()
-
-    sys.stdout.write('\n')
-
-def upgrade_loop(queue, backend):
-    plain_backend = backend.backend
-
-    while True:
-        obj_id = queue.get()
-        if obj_id is None:
-            break
-
-        meta = plain_backend.lookup(obj_id)
-        if meta.get('format_version', 0) == 2:
-            continue
-
-        # For important objects, we make a copy first (just to be safe)
-        if not obj_id.startswith('s3ql_data'):
-            plain_backend.copy(obj_id, 's3ql_pre2.13' + obj_id[4:])
-
-        # When reading passphrase objects, we have to use the
-        # "outer" password
-        if obj_id.startswith('s3ql_passphrase'):
-            data_pw = backend.passphrase
-            backend.passphrase = backend.fs_passphrase
-
-        meta = backend._convert_legacy_metadata(meta)
-        if meta['encryption'] == 'AES':
-            # Two statements to reduce likelihood of update races
-            size = rewrite_legacy_object(backend, obj_id)
-            queue.rewrote_size += size
-        else:
-            plain_backend.update_meta(obj_id, meta)
-
-        if obj_id.startswith('s3ql_passphrase'):
-            backend.passphrase = data_pw
-
-def rewrite_legacy_object(backend, obj_id):
-    with tempfile.TemporaryFile() as tmpfh:
-
-        # Read object
-        def do_read(fh):
-            tmpfh.seek(0)
-            tmpfh.truncate()
-            while True:
-                buf = fh.read(BUFSIZE)
-                if not buf:
-                    break
-                tmpfh.write(buf)
-            return fh.metadata
-
-        meta = backend.perform_read(do_read, obj_id)
-
-        # Write object
-        def do_write(fh):
-            tmpfh.seek(0)
-            while True:
-                buf = tmpfh.read(BUFSIZE)
-                if not buf:
-                    break
-                fh.write(buf)
-            return fh
-        out_fh = backend.perform_write(do_write, obj_id, meta)
-
-        return out_fh.get_obj_size()
+    try:
+        yield
+    finally:
+        metadata.DUMP_SPEC[2] = DUMP_SPEC_bak
+        metadata.create_tables = create_tables_bak
 
 if __name__ == '__main__':
     main(sys.argv[1:])
