@@ -14,6 +14,7 @@ import textwrap
 import hashlib
 import struct
 import hmac
+import random
 import inspect
 import ssl
 import os
@@ -141,7 +142,9 @@ def retry(method, _tracker=RateTracker(60)):
                     log.debug('retry_after is %.2f seconds', exc.retry_after)
                     interval = exc.retry_after
 
-            time.sleep(interval)
+            # Add some random variation to prevent flooding the
+            # server with too many concurrent requests.
+            time.sleep(interval * random.uniform(1, 1.5))
             waited += interval
             interval = min(5*60, 2*interval)
 
@@ -642,7 +645,7 @@ def get_proxy(ssl):
             proxy_port = 80
 
         proxy_host = hit.group(2)
-        log.info('Using CONNECT proxy %s:%d', proxy_host, proxy_port,
+        log.info('Using proxy %s:%d', proxy_host, proxy_port,
                  extra=LOG_ONCE)
         proxy = (proxy_host, proxy_port)
     else:
