@@ -161,10 +161,12 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
                or exc.status == 408)):
             return True
 
-        # Temporary workaround for https://bitbucket.org/nikratio/s3ql/issues/87.
-        # We still need to find a proper string
+        # Temporary workaround for
+        # https://bitbucket.org/nikratio/s3ql/issues/87 and
+        # https://bitbucket.org/nikratio/s3ql/issues/252
         elif (isinstance(exc, ssl.SSLError) and
-              str(exc).startswith('[SSL: BAD_WRITE_RETRY]')):
+              (str(exc).startswith('[SSL: BAD_WRITE_RETRY]') or
+               str(exc).startswith('[SSL: BAD_LENGTH]'))):
             return True
 
         return False
@@ -230,7 +232,14 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
         log.debug('started with %s, %s', prefix, start_after)
 
         keys_remaining = True
-        marker = self.prefix + start_after
+
+        # Without this, a call to list('foo') would result
+        # in *prefix* being longer than *marker* - which causes
+        # trouble for some S3 implementions (minio).
+        if start_after:
+            marker = self.prefix + start_after
+        else:
+            marker = ''
         prefix = self.prefix + prefix
 
         while keys_remaining:
@@ -273,7 +282,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
                         root.clear()
 
             except Exception as exc:
-                if is_temp_network_error(exc):
+                if is_temp_network_error(exc) or isinstance(exc, ssl.SSLError):
                     # We probably can't use the connection anymore
                     self.conn.disconnect()
                 raise
@@ -730,7 +739,7 @@ class Backend(AbstractBackend, metaclass=ABCDocstMeta):
             return read_response()
 
         except Exception as exc:
-            if is_temp_network_error(exc):
+            if is_temp_network_error(exc) or isinstance(exc, ssl.SSLError):
                 # We probably can't use the connection anymore
                 self.conn.disconnect()
             raise
