@@ -8,20 +8,22 @@ This work can be distributed under the terms of the GNU GPLv3.
 
 from .logging import logging # Ensure use of custom logger class
 from .database import NoSuchRowError
+import llfuse
+import sys
 
 log = logging.getLogger(__name__)
 
 CACHE_SIZE = 100
 ATTRIBUTES = ('mode', 'refcount', 'uid', 'gid', 'size', 'locked',
-              'rdev', 'atime', 'mtime', 'ctime', 'id')
+              'rdev', 'atime_ns', 'mtime_ns', 'ctime_ns', 'id')
 ATTRIBUTE_STR = ', '.join(ATTRIBUTES)
 UPDATE_ATTRS = ('mode', 'refcount', 'uid', 'gid', 'size', 'locked',
-              'rdev', 'atime', 'mtime', 'ctime')
+              'rdev', 'atime_ns', 'mtime_ns', 'ctime_ns')
 UPDATE_STR = ', '.join('%s=?' % x for x in UPDATE_ATTRS)
 
 MAX_INODE = 2 ** 32 - 1
 
-class _Inode(object):
+class _Inode:
     '''An inode with its attributes'''
 
     __slots__ = ATTRIBUTES + ('dirty', 'generation')
@@ -31,30 +33,32 @@ class _Inode(object):
         self.dirty = False
         self.generation = generation
 
-    # This allows access to all st_* attributes, even if they're
-    # not defined in the table
-    def __getattr__(self, key):
-        if key == 'st_nlink':
-            return self.refcount
-
-        elif key == 'st_blocks':
-            return (self.size + 511) // 512
-
-        elif key == 'st_ino':
-            return self.id
+    def entry_attributes(self):
+        attr = llfuse.EntryAttributes()
+        attr.st_nlink = self.refcount
+        attr.st_blocks = (self.size + 511) // 512
+        attr.st_ino = self.id
 
         # Timeout, can effectively be infinite since attribute changes
         # are only triggered by the kernel's own requests
-        elif key == 'attr_timeout' or key == 'entry_timeout':
-            return 3600
+        attr.attr_timeout = 3600
+        attr.entry_timeout = 3600
 
         # We want our blocksize for IO as large as possible to get large
         # write requests
-        elif key == 'st_blksize':
-            return 128 * 1024
+        attr.st_blksize = 128 * 1024
 
-        elif key.startswith('st_'):
-            return getattr(self, key[3:])
+        attr.st_mode = self.mode
+        attr.st_uid = self.uid
+        attr.st_gid = self.gid
+        attr.st_size = self.size
+        attr.st_rdev = self.rdev
+        attr.st_atime_ns = self.atime_ns
+        attr.st_mtime_ns = self.mtime_ns
+        attr.st_ctime_ns = self.ctime_ns
+        attr.generation = self.generation
+
+        return attr
 
     def __eq__(self, other):
         # Ill defined - should we compare the inode id or all the attributes?
@@ -78,6 +82,18 @@ class _Inode(object):
             object.__setattr__(self, 'dirty', True)
         object.__setattr__(self, name, value)
 
+    def __del__(self):
+        if not self.dirty:
+            return
+
+        # Force execution of sys.excepthook (exceptions raised
+        # by __del__ are ignored)
+        try:
+            raise RuntimeError('BUG ALERT: Dirty inode was destroyed!')
+        except RuntimeError:
+            exc_info = sys.exc_info()
+
+        sys.excepthook(*exc_info)
 
 class InodeCache(object):
     '''
@@ -133,10 +149,9 @@ class InodeCache(object):
     def __delitem__(self, inode):
         if self.db.execute('DELETE FROM inodes WHERE id=?', (inode,)) != 1:
             raise KeyError('No such inode')
-        try:
-            del self.attrs[inode]
-        except KeyError:
-            pass
+        inode = self.attrs.pop(inode, None)
+        if inode is not None:
+            inode.dirty = False
 
     def __getitem__(self, id_):
         try:
@@ -204,6 +219,8 @@ class InodeCache(object):
     def destroy(self):
         '''Flush all entries and empty cache'''
 
+        # Note: this method is currently also used for dropping the cache
+
         for i in range(len(self.cached_rows)):
             id_ = self.cached_rows[i]
             self.cached_rows[i] = None
@@ -235,9 +252,23 @@ class InodeCache(object):
                 else:
                     self.setattr(inode)
 
+    def drop(self):
+        '''Drop cache (after flushing)'''
+
+        self.destroy()
+
     def __del__(self):
-        if len(self.attrs) > 0:
+        if len(self.attrs) == 0:
+            return
+
+        # Force execution of sys.excepthook (exceptions raised
+        # by __del__ are ignored)
+        try:
             raise RuntimeError('InodeCache instance was destroyed without calling destroy()')
+        except RuntimeError:
+            exc_info = sys.exc_info()
+
+        sys.excepthook(*exc_info)
 
 
 
