@@ -24,12 +24,14 @@ def parse_args(args):
     parser.add_debug()
     parser.add_quiet()
     parser.add_version()
-    parser.add_fatal_warnings()
 
     parser.add_argument("mountpoint", metavar='<mountpoint>',
                         type=(lambda x: x.rstrip('/')),
                         help='Mount point of the file system to examine')
 
+    parser.add_argument("--raw", action="store_true", default=False,
+                          help="Do not pretty-print numbers")
+
     return parser.parse_args(args)
 
 def main(args=None):
@@ -41,29 +43,36 @@ def main(args=None):
     options = parse_args(args)
     setup_logging(options)
 
+    if options.raw:
+        pprint = lambda x: '%d bytes' % x
+    else:
+        pprint = pretty_print_size
+
     ctrlfile = assert_fs_owner(options.mountpoint, mountpoint=True)
 
     # Use a decent sized buffer, otherwise the statistics have to be
-    # calculated thee(!) times because we need to invoce getxattr
+    # calculated three(!) times because we need to invoke getxattr
     # three times.
     buf = llfuse.getxattr(ctrlfile, 's3qlstat', size_guess=256)
 
     (entries, blocks, inodes, fs_size, dedup_size,
-     compr_size, db_size, cache_used, cache_dirty) = struct.unpack('QQQQQQQQQ', buf)
+     compr_size, db_size, cache_cnt, cache_size, dirty_cnt,
+     dirty_size, removal_cnt) = struct.unpack('QQQQQQQQQQQQ', buf)
     p_dedup = dedup_size * 100 / fs_size if fs_size else 0
     p_compr_1 = compr_size * 100 / fs_size if fs_size else 0
     p_compr_2 = compr_size * 100 / dedup_size if dedup_size else 0
     print ('Directory entries:    %d' % entries,
            'Inodes:               %d' % inodes,
            'Data blocks:          %d' % blocks,
-           'Total data size:      %s' % pretty_print_size(fs_size),
+           'Total data size:      %s' % pprint(fs_size),
            'After de-duplication: %s (%.2f%% of total)'
-             % (pretty_print_size(dedup_size), p_dedup),
+             % (pprint(dedup_size), p_dedup),
            'After compression:    %s (%.2f%% of total, %.2f%% of de-duplicated)'
-             % (pretty_print_size(compr_size), p_compr_1, p_compr_2),
-           'Database size:        %s (uncompressed)' % pretty_print_size(db_size),
-           'Cache usage:          %s (dirty: %s)' % (pretty_print_size(cache_used),
-                                                     pretty_print_size(cache_dirty)),
+             % (pprint(compr_size), p_compr_1, p_compr_2),
+           'Database size:        %s (uncompressed)' % pprint(db_size),
+           'Cache size:           %s, %d entries' % (pprint(cache_size), cache_cnt),
+           'Cache size (dirty):   %s, %d entries' % (pprint(dirty_size), dirty_cnt),
+           'Queued object removals: %d' % (removal_cnt,),
            sep='\n')
 
 
