@@ -20,8 +20,7 @@ import unittest
 import subprocess
 import pytest
 
-
-@pytest.mark.usefixtures('s3ql_cmd_argv')
+@pytest.mark.usefixtures('s3ql_cmd_argv', 'pass_reg_output')
 class AdmTests(unittest.TestCase):
 
     def setUp(self):
@@ -37,7 +36,7 @@ class AdmTests(unittest.TestCase):
 
     def mkfs(self):
         proc = subprocess.Popen(self.s3ql_cmd_argv('mkfs.s3ql') +
-                                ['-L', 'test fs', '--max-obj-size', '500', '--fatal-warnings',
+                                ['-L', 'test fs', '--max-obj-size', '500',
                                  '--authfile', '/dev/null', '--cachedir', self.cache_dir,
                                  '--quiet', self.storage_url ],
                                 stdin=subprocess.PIPE, universal_newlines=True)
@@ -47,6 +46,8 @@ class AdmTests(unittest.TestCase):
         proc.stdin.close()
 
         self.assertEqual(proc.wait(), 0)
+        self.reg_output(r'^WARNING: Maximum object sizes less than '
+                        '1 MiB will degrade performance\.$', count=1)
 
     def test_passphrase(self):
         self.mkfs()
@@ -54,7 +55,7 @@ class AdmTests(unittest.TestCase):
         passphrase_new = 'sd982jhd'
 
         proc = subprocess.Popen(self.s3ql_cmd_argv('s3qladm') +
-                                [ '--quiet', '--fatal-warnings', '--log', 'none', '--authfile',
+                                [ '--quiet', '--log', 'none', '--authfile',
                                   '/dev/null', 'passphrase', self.storage_url ],
                                 stdin=subprocess.PIPE, universal_newlines=True)
 
@@ -86,7 +87,7 @@ class AdmTests(unittest.TestCase):
             fh.flush()
 
             proc = subprocess.Popen(self.s3ql_cmd_argv('fsck.s3ql') +
-                                    [ '--quiet', '--fatal-warnings', '--authfile', fh.name,
+                                    [ '--quiet', '--authfile', fh.name,
                                       '--cachedir', self.cache_dir, '--log', 'none', self.storage_url ],
                                     stdin=subprocess.PIPE, universal_newlines=True)
 
